@@ -36,6 +36,11 @@ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+/* Check on which OS we compile */
+#if defined(_WIN32) || defined(WIN32) || defined (__WIN32__) || defined(__CYGWIN__) || defined(__MINGW32__)
+#define OS_WIN
+#endif
+
 #include <limits.h>
 #include <stdlib.h>  /* strtod, strtol,... */
 #include <ctype.h> /* toupper, isspace */
@@ -67,8 +72,8 @@ static const char *parse_int_pair(const char *, const char *,
                                   pr_append_str *);
 
 static char  *parse_2_int_pair(const char*, char*, char,
-			       char, int*, int*, int*, int*,
-			       pr_append_str*); 
+                               char, int*, int*, int*, int*,
+                               pr_append_str*); 
 
 static void   parse_interval_list(const char *tag_name,
                                   const char *datum,
@@ -76,9 +81,9 @@ static void   parse_interval_list(const char *tag_name,
                                   pr_append_str *err);
 
 static void   parse_2_interval_list(const char *tag_name,
-				    char *datum,
-				    interval_array_t4 *interval_arr,
-				    pr_append_str *err);   
+                                    char *datum,
+                                    interval_array_t4 *interval_arr,
+                                    pr_append_str *err);   
 
 static int    parse_intron_list(char *, int *, int *);
 
@@ -124,7 +129,7 @@ extern double strtod();
 #define OVERWRITE_COMPARE_AND_MALLOC(TAG,T)        \
    if (COMPARE(TAG)) {                             \
        if (T) {                                    \
-	 free(T);				   \
+         free(T);                                   \
        }                                           \
        T = (char*) _rb_safe_malloc(datum_len + 1); \
        strcpy(T, datum);                           \
@@ -183,7 +188,7 @@ read_boulder_record(FILE *file_input,
                     seq_args *sa, 
                     pr_append_str *glob_err,  /* Really should be called fatal_parse_err */
                     pr_append_str *nonfatal_parse_err,
-		    pr_append_str *warnings,
+                    pr_append_str *warnings,
                     read_boulder_record_results *res) 
 {
   int line_len;
@@ -208,10 +213,10 @@ read_boulder_record(FILE *file_input,
        line except those begining "PRIMER_" or "P3_FILE_ID".
        Hint: strncomp returns 0 if both strings are equal */
      if (file_type == settings 
-	 && strncmp(s, "PRIMER_", 7) /* "s does not begin with 'PRIMER_'" */
-	 && strncmp(s, "P3_FILE_ID", 10) /* "s does not begin with
-					    'P3_FILE_ID'" */
-	 ) {
+         && strncmp(s, "PRIMER_", 7) /* "s does not begin with 'PRIMER_'" */
+         && strncmp(s, "P3_FILE_ID", 10) /* "s does not begin with
+                                            'P3_FILE_ID'" */
+         ) {
       continue;
     }
     /* Silently ignore all primer3plus tags */
@@ -271,10 +276,10 @@ read_boulder_record(FILE *file_input,
       COMPARE_INTERVAL_LIST("SEQUENCE_INTERNAL_EXCLUDED_REGION",
                               &sa->excl_internal2);
       if (COMPARE("SEQUENCE_OVERLAP_JUNCTION_LIST")) {
-	if (parse_intron_list(datum, sa->primer_overlap_junctions, 
-			      &sa->primer_overlap_junctions_count) == 0) {
+        if (parse_intron_list(datum, sa->primer_overlap_junctions, 
+                              &sa->primer_overlap_junctions_count) == 0) {
           pr_append_new_chunk(parse_err,
-			      "Error in SEQUENCE_PRIMER_OVERLAP_JUNCTION_LIST");
+                              "Error in SEQUENCE_PRIMER_OVERLAP_JUNCTION_LIST");
         }
         continue;
       }
@@ -334,11 +339,11 @@ read_boulder_record(FILE *file_input,
 
       /* COMPARE_FLOAT("PRIMER_MAX_SELF_END", pa->p_args.max_self_end); */
       /* NEW */ COMPARE_FLOAT_USE_FN("PRIMER_MAX_SELF_END", p3_set_gs_primer_self_end)
-	/* if (COMPARE("PRIMER_MAX_SELF_END")) {
-	  parse_double("PRIMER_MAX_SELF_END", datum, &tmp_double, parse_err);
-	  p3_set_gs_primer_self_end(pa, tmp_double);
-	  continue;
-	  } */
+        /* if (COMPARE("PRIMER_MAX_SELF_END")) {
+          parse_double("PRIMER_MAX_SELF_END", datum, &tmp_double, parse_err);
+          p3_set_gs_primer_self_end(pa, tmp_double);
+          continue;
+          } */
 
       COMPARE_FLOAT("PRIMER_MAX_SELF_ANY_TH", pa->p_args.max_self_any_th);
       COMPARE_FLOAT("PRIMER_MAX_SELF_END_TH", pa->p_args.max_self_end_th);
@@ -358,40 +363,40 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_MIN_QUALITY", pa->p_args.min_quality);
       COMPARE_INT("PRIMER_MIN_END_QUALITY", pa->p_args.min_end_quality);
       if (COMPARE("PRIMER_MIN_THREE_PRIME_DISTANCE")) {
-	parse_int("PRIMER_MIN_THREE_PRIME_DISTANCE", datum, &(min_three_prime_distance), parse_err);
-	/* check if specific tag also specified - error in this case */
-	if (min_3_prime_distance_specific == 1) {
-	  pr_append_new_chunk(glob_err,
+        parse_int("PRIMER_MIN_THREE_PRIME_DISTANCE", datum, &(min_three_prime_distance), parse_err);
+        /* check if specific tag also specified - error in this case */
+        if (min_3_prime_distance_specific == 1) {
+          pr_append_new_chunk(glob_err,
                               "Both PRIMER_MIN_THREE_PRIME_DISTANCE and PRIMER_{LEFT/RIGHT}_MIN_THREE_PRIME_DISTANCE specified");
-	} else {
-	  min_3_prime_distance_global = 1;
-	  /* set up individual flags */
-	  pa->min_left_three_prime_distance = min_three_prime_distance;
-	  pa->min_right_three_prime_distance = min_three_prime_distance;
-	}
-	continue;
+        } else {
+          min_3_prime_distance_global = 1;
+          /* set up individual flags */
+          pa->min_left_three_prime_distance = min_three_prime_distance;
+          pa->min_right_three_prime_distance = min_three_prime_distance;
+        }
+        continue;
       }
       if (COMPARE("PRIMER_MIN_LEFT_THREE_PRIME_DISTANCE")) {
-	parse_int("PRIMER_MIN_LEFT_THREE_PRIME_DISTANCE", datum, &(pa->min_left_three_prime_distance), parse_err);
-	/* check if global tag also specified - error in this case */
-	if (min_3_prime_distance_global == 1) {
-	  pr_append_new_chunk(glob_err,
+        parse_int("PRIMER_MIN_LEFT_THREE_PRIME_DISTANCE", datum, &(pa->min_left_three_prime_distance), parse_err);
+        /* check if global tag also specified - error in this case */
+        if (min_3_prime_distance_global == 1) {
+          pr_append_new_chunk(glob_err,
                               "Both PRIMER_MIN_THREE_PRIME_DISTANCE and PRIMER_{LEFT/RIGHT}_MIN_THREE_PRIME_DISTANCE specified");
-	} else {
-	  min_3_prime_distance_specific = 1;
-	}
-	continue;
+        } else {
+          min_3_prime_distance_specific = 1;
+        }
+        continue;
       }
       if (COMPARE("PRIMER_MIN_RIGHT_THREE_PRIME_DISTANCE")) {
-	parse_int("PRIMER_MIN_RIGHT_THREE_PRIME_DISTANCE", datum, &(pa->min_right_three_prime_distance), parse_err);
-	/* check if global tag also specified - error in this case */
-	if (min_3_prime_distance_global == 1) {
-	  pr_append_new_chunk(glob_err,
+        parse_int("PRIMER_MIN_RIGHT_THREE_PRIME_DISTANCE", datum, &(pa->min_right_three_prime_distance), parse_err);
+        /* check if global tag also specified - error in this case */
+        if (min_3_prime_distance_global == 1) {
+          pr_append_new_chunk(glob_err,
                               "Both PRIMER_MIN_THREE_PRIME_DISTANCE and PRIMER_{LEFT/RIGHT}_MIN_THREE_PRIME_DISTANCE specified");
-	} else {
-	  min_3_prime_distance_specific = 1;
-	}
-	continue;
+        } else {
+          min_3_prime_distance_specific = 1;
+        }
+        continue;
       }
       if (file_type == settings) {
         if (COMPARE("P3_FILE_ID")) continue;
@@ -406,14 +411,14 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_SEQUENCING_INTERVAL", pa->sequencing.interval);
       COMPARE_INT("PRIMER_SEQUENCING_ACCURACY", pa->sequencing.accuracy);
       if (COMPARE("PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION")) {
-	parse_int("PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION", datum, &pa->min_5_prime_overlap_of_junction, parse_err);
-	/* min_5_prime = 1; Removed 10/20/2010 */
-	continue;
+        parse_int("PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION", datum, &pa->min_5_prime_overlap_of_junction, parse_err);
+        /* min_5_prime = 1; Removed 10/20/2010 */
+        continue;
       }
       if (COMPARE("PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION")) {
-	parse_int("PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION", datum, &pa->min_3_prime_overlap_of_junction, parse_err);
-	/* min_3_prime = 1; Removed 10/20/2010 */
-	continue;
+        parse_int("PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION", datum, &pa->min_3_prime_overlap_of_junction, parse_err);
+        /* min_3_prime = 1; Removed 10/20/2010 */
+        continue;
       }
       COMPARE_AND_MALLOC("PRIMER_TASK", task_tmp);
       COMPARE_INT("PRIMER_PICK_RIGHT_PRIMER", pa->pick_right_primer);
@@ -440,24 +445,24 @@ read_boulder_record(FILE *file_input,
       COMPARE_INT("PRIMER_INTERNAL_MAX_NS_ACCEPTED", pa->o_args.num_ns_accepted);
       COMPARE_INT("PRIMER_INTERNAL_MIN_QUALITY", pa->o_args.min_quality);
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_ANY",
-		    pa->o_args.max_self_any);
+                    pa->o_args.max_self_any);
 
       /* COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_END", 
-	 pa->o_args.max_self_end); */
+         pa->o_args.max_self_end); */
       /* NEW  */ COMPARE_FLOAT_USE_FN("PRIMER_INTERNAL_MAX_SELF_END", 
-				      p3_set_gs_primer_internal_oligo_self_end);
+                                      p3_set_gs_primer_internal_oligo_self_end);
       /* if (COMPARE("PRIMER_INTERNAL_MAX_SELF_END")) { 
-	 parse_double("PRIMER_INTERNAL_MAX_SELF_END", datum, &tmp_double, parse_err);
-	 p3_set_gs_primer_internal_oligo_self_end(pa, tmp_double);
-	 continue;
-	 } */
+         parse_double("PRIMER_INTERNAL_MAX_SELF_END", datum, &tmp_double, parse_err);
+         p3_set_gs_primer_internal_oligo_self_end(pa, tmp_double);
+         continue;
+         } */
 
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_ANY_TH",
-			   pa->o_args.max_self_any_th);
+                           pa->o_args.max_self_any_th);
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_SELF_END_TH",
-				 pa->o_args.max_self_end_th);
+                                 pa->o_args.max_self_end_th);
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_HAIRPIN_TH",
-			       pa->o_args.max_hairpin_th);
+                               pa->o_args.max_hairpin_th);
       COMPARE_FLOAT("PRIMER_MAX_LIBRARY_MISPRIMING",
                     pa->p_args.max_repeat_compl);
       COMPARE_FLOAT("PRIMER_INTERNAL_MAX_LIBRARY_MISHYB",
@@ -468,11 +473,11 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_MAX_TEMPLATE_MISPRIMING",
                     pa->p_args.max_template_mispriming);
       COMPARE_FLOAT("PRIMER_MAX_TEMPLATE_MISPRIMING_TH",
-			  pa->p_args.max_template_mispriming_th);
+                          pa->p_args.max_template_mispriming_th);
       COMPARE_FLOAT("PRIMER_PAIR_MAX_TEMPLATE_MISPRIMING",
                     pa->pair_max_template_mispriming);
       COMPARE_FLOAT("PRIMER_PAIR_MAX_TEMPLATE_MISPRIMING_TH",
-		    pa->pair_max_template_mispriming_th);
+                    pa->pair_max_template_mispriming_th);
        /* Control interpretation of ambiguity codes in mispriming
           and mishyb libraries. */
       COMPARE_INT("PRIMER_LIB_AMBIGUITY_CODES_CONSENSUS",
@@ -505,35 +510,37 @@ read_boulder_record(FILE *file_input,
       }
       if (COMPARE("P3_COMMENT")) continue;
       COMPARE_FLOAT("PRIMER_MAX_END_STABILITY", pa->max_end_stability);
-      COMPARE_INT("PRIMER_LOWERCASE_MASKING",
-                  pa->lowercase_masking);
+      COMPARE_INT("PRIMER_LOWERCASE_MASKING", pa->lowercase_masking);
 
       /* added by T. Koressaar */
       COMPARE_INT("PRIMER_THERMODYNAMIC_OLIGO_ALIGNMENT", pa->thermodynamic_oligo_alignment);
       COMPARE_INT("PRIMER_THERMODYNAMIC_TEMPLATE_ALIGNMENT", pa->thermodynamic_template_alignment);
       if (COMPARE("PRIMER_THERMODYNAMIC_PARAMETERS_PATH")) {
-        if (thermodynamic_params_path == NULL) {
-          thermodynamic_params_path = (char*) _rb_safe_malloc(datum_len + 1);
-          strcpy(thermodynamic_params_path, datum);
-          thermodynamic_path_changed = 1;
+        thermodynamic_params_path = (char*) _rb_safe_malloc(datum_len + 1);
+        strcpy(thermodynamic_params_path, datum);
+        if (thermodynamic_params_path[strlen(thermodynamic_params_path) - 1] == '\n') {
+	  thermodynamic_params_path[strlen(thermodynamic_params_path) - 1] = '\0';
         }
-        /* check if path changes */
-	else if (strcmp(thermodynamic_params_path, datum)) {
-	  free(thermodynamic_params_path);
-	  thermodynamic_params_path = (char*) _rb_safe_malloc(datum_len + 1); 
-	  strcpy(thermodynamic_params_path, datum);
-	  thermodynamic_path_changed = 1;
+
+        thal_results o;
+	if (thal_load_parameters(thermodynamic_params_path, &pa->thermodynamic_parameters, &o) == -1) {
+	  pr_append_new_chunk(glob_err, o.msg);
+	}
+        if (get_thermodynamic_values(&pa->thermodynamic_parameters, &o)) {
+          pr_append_new_chunk(glob_err, o.msg);
+        }
+	free(thermodynamic_params_path);
+	continue;
+      }
+      if (COMPARE("PRIMER_MASK_KMERLIST_PATH")) {
+        if (kmer_lists_path == NULL) {
+          kmer_lists_path = (char*) _rb_safe_malloc(datum_len + 1);
+          strcpy(kmer_lists_path, datum);
         }
-        continue;
-      }  
-        if (COMPARE("PRIMER_MASK_KMERLIST_PATH")) {
-           if (kmer_lists_path == NULL) {
-               kmer_lists_path = (char*) _rb_safe_malloc(datum_len + 1);
-               strcpy(kmer_lists_path, datum);
-           }
         continue;
       }
       /* added by A. Untergasser */
+      COMPARE_INT("PRIMER_SECONDARY_STRUCTURE_ALIGNMENT", pa->show_secondary_structure_alignment);
       OVERWRITE_COMPARE_AND_MALLOC("PRIMER_MUST_MATCH_FIVE_PRIME", pa->p_args.must_match_five_prime);
       OVERWRITE_COMPARE_AND_MALLOC("PRIMER_MUST_MATCH_THREE_PRIME", pa->p_args.must_match_three_prime);
       OVERWRITE_COMPARE_AND_MALLOC("PRIMER_INTERNAL_MUST_MATCH_FIVE_PRIME", pa->o_args.must_match_five_prime);
@@ -561,7 +568,7 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_WT_TEMPLATE_MISPRIMING",
                     pa->p_args.weights.template_mispriming);
       COMPARE_FLOAT("PRIMER_WT_TEMPLATE_MISPRIMING_TH",
-		                        pa->p_args.weights.template_mispriming_th);
+                                        pa->p_args.weights.template_mispriming_th);
       COMPARE_FLOAT("PRIMER_WT_MASK_FAILURE_RATE",
                                               pa->p_args.weights.failure_rate);                        
       COMPARE_FLOAT("PRIMER_INTERNAL_WT_TM_GT", pa->o_args.weights.temp_gt);
@@ -580,7 +587,7 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_INTERNAL_WT_SEQ_QUAL", pa->o_args.weights.seq_quality);
       COMPARE_FLOAT("PRIMER_INTERNAL_WT_END_QUAL", pa->o_args.weights.end_quality);
       COMPARE_FLOAT("PRIMER_WT_TEMPLATE_MISPRIMING_TH",
-		    pa->o_args.weights.template_mispriming_th);
+                    pa->o_args.weights.template_mispriming_th);
       COMPARE_FLOAT("PRIMER_PAIR_WT_PR_PENALTY", 
                     pa->pr_pair_weights.primer_quality);
       COMPARE_FLOAT("PRIMER_PAIR_WT_IO_PENALTY",
@@ -592,9 +599,9 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_PAIR_WT_COMPL_END",
                     pa->pr_pair_weights.compl_end);
       COMPARE_FLOAT("PRIMER_PAIR_WT_COMPL_ANY_TH",
-		    pa->pr_pair_weights.compl_any_th);
+                    pa->pr_pair_weights.compl_any_th);
       COMPARE_FLOAT("PRIMER_PAIR_WT_COMPL_END_TH",
-		    pa->pr_pair_weights.compl_end_th);
+                    pa->pr_pair_weights.compl_end_th);
       COMPARE_FLOAT("PRIMER_PAIR_WT_PRODUCT_TM_LT",
                     pa->pr_pair_weights.product_tm_lt);
       COMPARE_FLOAT("PRIMER_PAIR_WT_PRODUCT_TM_GT",
@@ -608,21 +615,31 @@ read_boulder_record(FILE *file_input,
       COMPARE_FLOAT("PRIMER_PAIR_WT_TEMPLATE_MISPRIMING",
                     pa->pr_pair_weights.template_mispriming);
       COMPARE_FLOAT("PRIMER_PAIR_WT_TEMPLATE_MISPRIMING_TH",
-		    pa->pr_pair_weights.template_mispriming_th);
+                    pa->pr_pair_weights.template_mispriming_th);
       if (COMPARE("PRIMER_MASK_TEMPLATE")) {
-                    parse_int("PRIMER_MASK_TEMPLATE", datum, &pa->mask_template, parse_err);  
-                    pa->lowercase_masking = pa->mask_template;
+                    parse_int("PRIMER_MASK_TEMPLATE", datum, &pa->mask_template, parse_err);
+                    /* Check in case: PRIMER_LOWERCASE_MASKING=1 and PRIMER_MASK_TEMPLATE=0 */
+#if !defined(OS_WIN)
+                    if (pa->mask_template) {
+                        pa->lowercase_masking = pa->mask_template;
+                    }
                     pa->masking_parameters_changed = pa->mask_template;
                     continue;
+#else
+                    if (pa->mask_template) {    
+                        pr_append_new_chunk(glob_err, "PRIMER_MASK_TEMPLATE not supported on Windows");
+                    }
+                    continue;
+#endif
       }
       COMPARE_FLOAT("PRIMER_MASK_FAILURE_RATE", pa->mp.failure_rate);
       COMPARE_INT("PRIMER_MASK_5P_DIRECTION", pa->mp.nucl_masked_in_5p_direction);
       COMPARE_INT("PRIMER_MASK_3P_DIRECTION", pa->mp.nucl_masked_in_3p_direction);
       if (COMPARE("PRIMER_MASK_KMERLIST_PREFIX")) {
          if(pa->mp.list_prefix == NULL){
-	    pa->mp.list_prefix = (char*) _rb_safe_malloc(datum_len + 1);
-	    strcpy(pa->mp.list_prefix, datum);
-	    pa->masking_parameters_changed = 1;
+            pa->mp.list_prefix = (char*) _rb_safe_malloc(datum_len + 1);
+            strcpy(pa->mp.list_prefix, datum);
+            pa->masking_parameters_changed = 1;
          }  else if (strcmp(pa->mp.list_prefix, datum)){
              free(pa->mp.list_prefix);
              pa->mp.list_prefix = (char*) _rb_safe_malloc(datum_len + 1);
@@ -637,9 +654,9 @@ read_boulder_record(FILE *file_input,
         
     /*  Complain about unrecognized tags */
      if (*strict_tags == 1) {
-	pr_append_new_chunk(glob_err, "Unrecognized tag: ");
-	pr_append(glob_err, s);
-	fprintf(stderr, "Unrecognized tag: %s\n", s);
+        pr_append_new_chunk(glob_err, "Unrecognized tag: ");
+        pr_append(glob_err, s);
+        fprintf(stderr, "Unrecognized tag: %s\n", s);
      }
   }  /* while ((s = p3_read_line(stdin)) != NULL && strcmp(s,"=")) { */
 
@@ -683,7 +700,7 @@ read_boulder_record(FILE *file_input,
       pa->primer_task = generic;
     } else if (!strcmp_nocase(task_tmp, "pick_detection_primers")) {
       pa->primer_task = generic; /* Deliberate duplication for
-				    backward compatibility. */
+                                    backward compatibility. */
     } else if (!strcmp_nocase(task_tmp, "pick_cloning_primers")) {
       pa->primer_task = pick_cloning_primers;
     } else if (!strcmp_nocase(task_tmp, "pick_discriminative_primers")) {
@@ -770,9 +787,9 @@ read_boulder_record(FILE *file_input,
      if ((min_3_prime || min_5_prime) && (sa->primer_overlap_junctions_count == 0)) {
      pr_append_new_chunk(warnings,
                          "SEQUENCE_OVERLAP_JUNCTION_LIST not given, but "
-			 "PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION or "
-			 "PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION specified");
-			 } */
+                         "PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION or "
+                         "PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION specified");
+                         } */
 
   return 1;
 }
@@ -784,23 +801,23 @@ read_boulder_record(FILE *file_input,
 
 static void
 pr_append2(pr_append_str *err,
-	   const char* s1, 
-	   const char* s2) {
+           const char* s1, 
+           const char* s2) {
     pr_append_new_chunk(err, s1);
     pr_append(err, s2);
 }  
 
 int 
 read_p3_file(const char *file_name,
-	     const p3_file_type expected_file_type,
-	     int echo_output,
-	     int strict_tags,
-	     p3_global_settings *pa, 
-	     seq_args *sa,
-	     pr_append_str *fatal_err,
-	     pr_append_str *nonfatal_err,
-	     pr_append_str *warnings,
-	     read_boulder_record_results *read_boulder_record_res) 
+             const p3_file_type expected_file_type,
+             int echo_output,
+             int strict_tags,
+             p3_global_settings *pa, 
+             seq_args *sa,
+             pr_append_str *fatal_err,
+             pr_append_str *nonfatal_err,
+             pr_append_str *warnings,
+             read_boulder_record_results *read_boulder_record_res) 
 {
   FILE *file;
   int ret_par = 0;
@@ -825,15 +842,15 @@ read_p3_file(const char *file_name,
   line1 = p3_read_line(file);
   if (!line1) {
     pr_append2(fatal_err, 
-	       "Settings file is empty: ", 
-	       file_name);
+               "Settings file is empty: ", 
+               file_name);
     return ret_par;
   }
   if ((strcmp(line1,"Primer3 File - http://primer3.org") != 0) &&
       (strcmp(line1,"Primer3 File - http://primer3.sourceforge.net") != 0)) {
       pr_append2(fatal_err,
                  "First line must be \"Primer3 File - http://primer3.org\" in ",
-		 file_name);
+                 file_name);
       return ret_par;
   }
 
@@ -841,8 +858,8 @@ read_p3_file(const char *file_name,
   line2 = p3_read_line(file);
   if (!line2) {
     pr_append2(fatal_err, 
-	       "Incorrect file format (too few lines) in ", 
-	       file_name);
+               "Incorrect file format (too few lines) in ", 
+               file_name);
     return ret_par;
   }
   if (!strcmp(line2,"P3_FILE_TYPE=all_parameters")) {
@@ -853,7 +870,7 @@ read_p3_file(const char *file_name,
     file_type = settings;
   } else {
     pr_append2(fatal_err, 
-	       "Unknown file type in at line 2 (line2='", line2);
+               "Unknown file type in at line 2 (line2='", line2);
     pr_append(fatal_err, "') in ");
     pr_append(fatal_err, file_name);
     return ret_par;
@@ -867,34 +884,34 @@ read_p3_file(const char *file_name,
   line3 = p3_read_line(file);
   if (!line3) {
     pr_append2(fatal_err, 
-	       "Incorrect file format (too few lines) in ", 
-	       file_name);
+               "Incorrect file format (too few lines) in ", 
+               file_name);
       return ret_par;
   }
   if (strcmp(line3, "")) {
     pr_append2(fatal_err, "Line 3 must be empty in ",
-	       file_name);
+               file_name);
     return ret_par;
   }
 
   /* Check if the file type matches the expected type */
   if (file_type != expected_file_type){
     pr_append_new_chunk(nonfatal_err, 
-			"Unexpected P3 file type parsed");
+                        "Unexpected P3 file type parsed");
   }
 
   /* read the file */
   ret_par = read_boulder_record(file, &strict_tags, &io_version, 
-				echo_output, expected_file_type,
-				pa, sa, fatal_err, 
-				nonfatal_err, warnings, 
-				read_boulder_record_res);
+                                echo_output, expected_file_type,
+                                pa, sa, fatal_err, 
+                                nonfatal_err, warnings, 
+                                read_boulder_record_res);
   if(pa->mask_template){
     pa->lowercase_masking=pa->mask_template;
   }
   if (echo_output) printf("P3_SETTINGS_FILE_END=\n");
   if (file) fclose(file);
-         
+  
   return ret_par;
 }
 
@@ -1044,11 +1061,11 @@ parse_interval_list(const char *tag_name,
  */
 static char *
 parse_2_int_pair(const char    *tag_name, char *datum,
-		 char          sep,              /* The separator between 2 numbers, e.g. ',' or '-'. */
-		 char          sep2,             /* Separator between 2 intervals */
-		 int           *out1, int *out2, 
-		 int           *out3, int *out4, /* The 4 integers. */
-		 pr_append_str *err)             /* Error messages. */
+                 char          sep,              /* The separator between 2 numbers, e.g. ',' or '-'. */
+                 char          sep2,             /* Separator between 2 intervals */
+                 int           *out1, int *out2, 
+                 int           *out3, int *out4, /* The 4 integers. */
+                 pr_append_str *err)             /* Error messages. */
 {
     char *nptr, *tmp;
     long tlong;
@@ -1089,7 +1106,7 @@ parse_2_int_pair(const char    *tag_name, char *datum,
       }
       *out2 = tlong;
       if (nptr == tmp) {
-	tag_syntax_error(tag_name, datum, err);
+        tag_syntax_error(tag_name, datum, err);
         return NULL;
       }
     }
@@ -1112,7 +1129,7 @@ parse_2_int_pair(const char    *tag_name, char *datum,
       }
       *out3 = tlong;
       if (nptr == tmp) {
-	tag_syntax_error(tag_name, datum, err);
+        tag_syntax_error(tag_name, datum, err);
         return NULL;
       }
     }
@@ -1135,7 +1152,7 @@ parse_2_int_pair(const char    *tag_name, char *datum,
       }
       *out4 = tlong;
       if (nptr == tmp) {
-	tag_syntax_error(tag_name, datum, err);
+        tag_syntax_error(tag_name, datum, err);
         return NULL;
       }
     }
@@ -1156,9 +1173,9 @@ parse_2_int_pair(const char    *tag_name, char *datum,
 
 static void
 parse_2_interval_list(const char *tag_name,
-		      char *datum,
-		      interval_array_t4 *interval_arr,
-		      pr_append_str *err)
+                      char *datum,
+                      interval_array_t4 *interval_arr,
+                      pr_append_str *err)
 {
   char *p = datum;
   int i1, i2, i3, i4;
@@ -1185,8 +1202,8 @@ parse_2_interval_list(const char *tag_name,
 
 static int
 parse_intron_list(char *s,
-		  int *list,
-		  int *count) 
+                  int *list,
+                  int *count) 
 {
   long t;
   char *p, *q;
@@ -1200,7 +1217,7 @@ parse_intron_list(char *s,
     if (q == p) {
       while (*q != '\0') {
         if (!isspace(*q)) {
-	  *count = 0;
+          *count = 0;
           return 0; 
         }
         q++;
